# coding: utf-8
"""
GUI styling class
"""



class DummyFallback(object):
    """
    Dummy style for default stye fallback.
    """
    def __getitem__(self, name):
        return None
    
    def __call__(self, name, widget, default=None, suffix=""):
        return None
    
    def __nonzero__(self):
        return False

class Style(dict):
    def __init__(self, fallback=None, **kwargs):
        dict.__init__(self, **kwargs)
        self.fallback = fallback or DummyFallback()
        
   # def __getitem__(self, name):
        #return self.get(name, None) or self.fallback[name]
    
    def __call__(self, name, widget, default=None, suffix=None):
        """
        get the attribute with name ``name`` prefixed with the classname of
        ``ẁidget``\ . If this does not exist, it will pass the call to
        :attr:``fallback``
        """
        #! improve suffix handling
        if suffix is None:
            suffix = widget.get_suffix(name)
        if suffix:
            # use False as default to enable overwriting with None
            # eg: "image: ...; image_active: None"
            # if suffix fails, try without
            style = self("%s_%s" % (name, suffix), widget, False, False)
            if style is not False:
                return style
        cls = widget.__class__
        first_name = "%s_%s" % ((widget.style_class_name or
                                 cls.__name__).lower(), name)
                                 
        # styles can also be overwritten with ``None``
        retval = None
        return_ = False
        if first_name in self:
            retval = self[first_name]
            return_ = True
        elif name in self:
            retval = self[name]
            return_ = True
        if return_:
            if callable(retval):
                retval = retval(widget=widget)
            return retval  
        return self.fallback(name, widget) or default
    
        #return (self.get(first_name, self.get(name, None)) or
        #        self.fallback(name, widget) or default)
    

